@@ -1,11 +1,10 @@
-use 5.008;    # utf8
+use 5.006;
 use strict;
 use warnings;
-use utf8;
 
 package Dist::Zilla::Util::ExpandINI::Writer;
 
-our $VERSION = '0.001003';
+our $VERSION = '0.002000';
 
 # ABSTRACT: An order-preserving INI Writer
 
@@ -41,7 +40,7 @@ sub preprocess_input {
   for my $ini_record ( @{$input_data} ) {
     $i++;
     if ( $ini_record->{name} and '_' eq $ini_record->{name} ) {
-      push @out, '_', $ini_record->{lines};
+      push @out, '_', $ini_record;
       next;
     }
     if ( not $ini_record->{package} ) {
@@ -51,7 +50,7 @@ sub preprocess_input {
     if ( $ini_record->{name} and $ini_record->{package} ne $ini_record->{name} ) {
       $kn .= q[ / ] . $ini_record->{name};
     }
-    push @out, $kn, $ini_record->{lines};
+    push @out, $kn, $ini_record;
     next;
   }
   return \@out;
@@ -65,7 +64,7 @@ sub validate_input {
   my @input_copy = @{$input};
 
   while (@input_copy) {
-    my ( $name, $props ) = splice @input_copy, 0, 2;
+    my ( $name, $ini_record ) = splice @input_copy, 0, 2;
 
     if ( $seen{$name}++ ) {
       Carp::croak "multiple declarations found of $name";
@@ -74,7 +73,7 @@ sub validate_input {
     Carp::croak "illegal section name '$name'"
       if not $self->is_valid_section_name($name);
 
-    my @props_copy = @{$props};
+    my @props_copy = @{ $ini_record->{lines} };
 
     while (@props_copy) {
       my ( $property, $value ) = splice @props_copy, 0, 2;
@@ -89,6 +88,14 @@ sub validate_input {
   }
   return;
 }
+
+sub stringify_section_data {
+  my ( $self, $ini_record ) = @_;
+  my $output = q[];
+  $output .= q[;] . $_ . qq[\n] for @{ $ini_record->{comment_lines} };
+  $output .= $self->SUPER::stringify_section_data( $ini_record->{lines} );
+  return $output;
+}
 1;
 
 __END__
@@ -103,13 +110,13 @@ Dist::Zilla::Util::ExpandINI::Writer - An order-preserving INI Writer
 
 =head1 VERSION
 
-version 0.001003
+version 0.002000
 
 =for Pod::Coverage is_valid_section_name
 
 =head1 AUTHOR
 
-Kent Fredric <kentfredric@gmail.com>
+Kent Fredric <kentnl@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
